@@ -1,6 +1,8 @@
 /*
 
   Copyright (C) 2003  Razvan Cojocaru <razvanco@gmx.net>
+   XML-RPC/Context ID code contributed by Eamon Millman / PCI Geomatics
+  <millman@pcigeomatics.com>
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,6 +29,7 @@
 #include <wx/strconv.h>
 #include <wx/fontmap.h>
 #include <wx/treectrl.h>
+
 #include <assert.h>
 
 #include <bitfiddle.inl>
@@ -53,18 +56,6 @@ private:
 } // namespace
 
 
-#ifdef wxUSE_UNICODE
-
-#	define CURRENT_CHAR_STRING(x) \
-	wxString(reinterpret_cast<const char *>(x), wxConvISO8859_1)
-
-#else
-
-#	define CURRENT_CHAR_STRING(x) \
-	wxString(reinterpret_cast<const char *>(x))
-
-#endif
-
 // Big-enough buffer size for use with various routines.
 #define BUF_SIZE 4096
 
@@ -95,7 +86,8 @@ private:
 
 CHMFile::CHMFile()
 	: _chmFile(NULL), _home(wxT("/"))
-{}
+{
+}
 
 
 CHMFile::CHMFile(const wxString& archiveName)
@@ -125,6 +117,7 @@ bool CHMFile::LoadCHM(const wxString&  archiveName)
 
 	_filename = archiveName;	
 	GetArchiveInfo();
+	LoadContextIDs();
 
 	return true;
 }
@@ -138,6 +131,7 @@ void CHMFile::CloseCHM()
 
 	chm_close(_chmFile);
 	
+	_cidMap.clear();
 	_chmFile = NULL;
 	_home = wxT("/");
 	_filename = _home = _topicsFile = _indexFile 
@@ -208,6 +202,91 @@ bool CHMFile::GetIndex(CHMListCtrl* toBuild)
 }
 
 
+bool CHMFile::LoadContextIDs()
+{
+	chmUnitInfo ivb_ui, strs_ui;
+
+	_cidMap.clear();
+
+	// make sure what we need is there. 
+	// #IVB has list of context ID's and #STRINGS offsets to file names.
+	if( chm_resolve_object(_chmFile, "/#IVB", &ivb_ui ) != 
+				CHM_RESOLVE_SUCCESS ||
+		chm_resolve_object(_chmFile, "/#STRINGS", &strs_ui) != 
+				CHM_RESOLVE_SUCCESS )
+		return false; // failed to find internal files
+	
+	UCharPtr ivb_buf(new unsigned char[ivb_ui.length]);
+	u_int64_t ivb_len = 0;
+
+	if((ivb_len = chm_retrieve_object(_chmFile, &ivb_ui, 
+					  ivb_buf.get(), 0, 
+					  ivb_ui.length)) == 0 )
+		return false; // failed to retrieve data
+
+	// always odd (DWORD + 2(n)*DWORD, so make even
+	ivb_len = ivb_len/sizeof(u_int32_t) - 1; 
+	
+	if( ivb_len % 2 != 0 )
+		return false; // we retrieved unexpected data from the file.
+
+	u_int32_t *ivbs = new u_int32_t[ivb_len];
+	int j = 4; // offset to exclude first DWORD
+	
+	// convert our DWORDs to numbers
+	for( unsigned int i = 0; i < ivb_len; i++ )
+	{
+		ivbs[i] = UINT32ARRAY(ivb_buf.get() + j);
+		j+=4; // step to the next DWORD
+	}
+
+	UCharPtr strs_buf(new unsigned char[strs_ui.length]);
+	u_int64_t strs_len = 0;
+
+	if( (strs_len = chm_retrieve_object(_chmFile, &strs_ui, 
+					    strs_buf.get(), 
+					    0, strs_ui.length)) == 0 ) {
+		delete[] ivbs;
+		return false; // failed to retrieve data
+	}
+
+	for( unsigned int i = 0; i < ivb_len; i+=2 )
+	{	// context-IDs as KEY, fileName from #STRINGS as VALUE
+		_cidMap[ivbs[i]] = CURRENT_CHAR_STRING(
+			strs_buf.get() + ivbs[i+1]);
+	}
+
+	delete[] ivbs;
+	
+	// everything went well!
+	return true;
+}
+
+bool CHMFile::IsValidCID( const int contextID )
+{
+	if(_cidMap.empty())
+		return FALSE;
+	CHMIDMap::iterator itr = _cidMap.find( contextID );
+	if( itr == _cidMap.end() )
+		return FALSE;
+
+	return TRUE;
+}
+
+wxString CHMFile::GetPageByCID( const int contextID )
+{
+	if(_cidMap.empty())
+		return wxT("/");
+
+	CHMIDMap::iterator itr = _cidMap.find( contextID );
+	// make sure the key/value pair is valid
+	if(itr == _cidMap.end() ) 
+		return wxT("/");
+	
+	return wxString(wxT("/")) + itr->second;
+}
+
+
 bool CHMFile::IndexSearch(const wxString& text, bool wholeWords, 
 			  bool titlesOnly, CHMSearchResults *results)
 {
@@ -246,17 +325,14 @@ bool CHMFile::IndexSearch(const wxString& text, bool wholeWords,
 		return false;
 	}
 
-	u_int32_t* cursor32 = reinterpret_cast<u_int32_t*>(header + 0x14);
-	u_int32_t node_offset = *cursor32;
-	FIXENDIAN32(node_offset);
+	unsigned char* cursor32 = header + 0x14;
+	u_int32_t node_offset = UINT32ARRAY(cursor32);
 
-	cursor32 = reinterpret_cast<u_int32_t*>(header + 0x2e);
-	u_int32_t node_len = *cursor32;
-	FIXENDIAN32(node_len);
+	cursor32 = header + 0x2e;
+	u_int32_t node_len = UINT32ARRAY(cursor32);
 
-	u_int16_t* cursor16 = reinterpret_cast<u_int16_t*>(header + 0x18);
-	u_int16_t tree_depth = *cursor16;
-	FIXENDIAN16(tree_depth);
+	unsigned char* cursor16 = header + 0x18;
+	u_int16_t tree_depth = UINT16ARRAY(cursor16);
 
 	unsigned char word_len, pos;
 	wxString word;
@@ -272,15 +348,13 @@ bool CHMFile::IndexSearch(const wxString& text, bool wholeWords,
 		return false;
 
 	do {
-		
 		// got a leaf node here.
 		if(::chm_retrieve_object(_chmFile, &ui, buffer.get(), 
 					 node_offset, node_len) == 0)
 			return false;
 
-		cursor16 = reinterpret_cast<u_int16_t *>(buffer.get() + 6);
-		free_space = *cursor16;
-		FIXENDIAN16(free_space);
+		cursor16 = buffer.get() + 6;
+		free_space = UINT16ARRAY(cursor16);
 
 		i = sizeof(u_int32_t) + sizeof(u_int16_t) + sizeof(u_int16_t);
 		u_int64_t wlc_count, wlc_size;
@@ -309,18 +383,15 @@ bool CHMFile::IndexSearch(const wxString& text, bool wholeWords,
 			wlc_count = be_encint(buffer.get() + i, encsz);
 			i += encsz;
 		
-			cursor32 = reinterpret_cast<u_int32_t *>(
-				buffer.get() + i);
-			wlc_offset = *cursor32;
-			FIXENDIAN32(wlc_offset);
+			cursor32 = buffer.get() + i;
+			wlc_offset = UINT32ARRAY(cursor32);
 
 			i += sizeof(u_int32_t) + sizeof(u_int16_t);
 			wlc_size =  be_encint(buffer.get() + i, encsz);
 			i += encsz;
 
-			cursor32 = reinterpret_cast<u_int32_t*>(buffer.get());
-			node_offset = *cursor32;
-			FIXENDIAN32(node_offset);
+			cursor32 = buffer.get();
+			node_offset = UINT32ARRAY(cursor32);
 		
 			if(!title && titlesOnly)
 				continue;
@@ -350,9 +421,10 @@ bool CHMFile::IndexSearch(const wxString& text, bool wholeWords,
 						  word.Mid(0, text.Length()))
 					  < -1)
 					break;
-				   
-						  
 			}
+
+			if(results->size() >= MAX_SEARCH_RESULTS)
+				break;
 		}	
 	} while(!wholeWords && word.StartsWith(text.c_str()) && node_offset);
 
@@ -396,8 +468,8 @@ u_int32_t CHMFile::GetLeafNodeOffset(const wxString& text,
 				     chmUnitInfo *ui)
 {
 	u_int32_t test_offset = 0;
-	u_int16_t* cursor16;
-	u_int32_t* cursor32;
+	unsigned char* cursor16;
+	unsigned char* cursor32;
 	unsigned char word_len, pos;
 	u_int32_t i = sizeof(u_int16_t);
 	UCharPtr buffer(new unsigned char[buffSize]);
@@ -415,9 +487,8 @@ u_int32_t CHMFile::GetLeafNodeOffset(const wxString& text,
 					 initialOffset, buffSize) == 0)
 			return 0;
 
-		cursor16 = reinterpret_cast<u_int16_t*>(buffer.get());
-		u_int16_t free_space = *cursor16;
-		FIXENDIAN16(free_space);
+		cursor16 = buffer.get();
+		u_int16_t free_space = UINT16ARRAY(cursor16);
 
 		while(i < buffSize - free_space) {
 
@@ -437,10 +508,8 @@ u_int32_t CHMFile::GetLeafNodeOffset(const wxString& text,
 			delete[] wrd_buf;
 
 			if(text.CmpNoCase(word) <= 0) {
-				cursor32 = reinterpret_cast<u_int32_t*>(
-					buffer.get() + i + word_len + 1);
-				initialOffset = *cursor32;
-				FIXENDIAN32(initialOffset);
+				cursor32 = buffer.get() + i + word_len + 1;
+				initialOffset = UINT32ARRAY(cursor32);
 				break;
 			}
 
@@ -470,7 +539,7 @@ bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 	u_int64_t index = 0, count;
 	size_t length, off = 0;
 	UCharPtr buffer(new unsigned char[wlc_size]);
-	u_int32_t *cursor32;
+	unsigned char *cursor32;
 	u_int32_t stroff, urloff;
 
 #define TOPICS_ENTRY_LEN 16
@@ -497,10 +566,9 @@ bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 					 index * 16, TOPICS_ENTRY_LEN) == 0)
 			return false;
 
-		cursor32 = reinterpret_cast<u_int32_t *>(entry + 4);
+		cursor32 = entry + 4;
 		combuf[COMMON_BUF_LEN - 1] = 0;
-		stroff = *cursor32;
-		FIXENDIAN32(stroff);
+		stroff = UINT32ARRAY(cursor32);
 
 		wxString topic;
 
@@ -524,17 +592,15 @@ bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 #endif
 		}
 	      
-		cursor32 = reinterpret_cast<u_int32_t *>(entry + 8);
-		urloff = *cursor32;
-		FIXENDIAN32(urloff);
+		cursor32 = entry + 8;
+		urloff = UINT32ARRAY(cursor32);
 
 		if(::chm_retrieve_object(_chmFile, uitbl, combuf, 
 					 urloff, 12) == 0)
 			return false;
 
-		cursor32 = reinterpret_cast<u_int32_t*>(combuf + 8);
-		urloff = *cursor32;
-		FIXENDIAN32(urloff);
+		cursor32 = combuf + 8;
+		urloff = UINT32ARRAY(cursor32);
 
 		if(::chm_retrieve_object(_chmFile, urlstr, combuf, 
 					 urloff + 8, COMMON_BUF_LEN - 1) == 0)
@@ -544,8 +610,11 @@ bool CHMFile::ProcessWLC(u_int64_t wlc_count, u_int64_t wlc_size,
 
 		wxString url = CURRENT_CHAR_STRING(combuf);
 
-		if(!url.IsEmpty() && !topic.IsEmpty())
+		if(!url.IsEmpty() && !topic.IsEmpty()) {
+			if(results->size() >= MAX_SEARCH_RESULTS)
+				return true;
 			(*results)[url] = topic;
+		}
 
 		count = sr_int(buffer.get() + off, &wlc_bit, cs, cr, length);
 		off += length;
@@ -697,7 +766,9 @@ bool CHMFile::InfoFromSystem()
 	chmUnitInfo ui;
 	
 	int index = 0;
-	u_int16_t *cursor = NULL;
+	unsigned char* cursor = NULL;
+	u_int16_t value = 0;
+
 	long size = 0;
 	long cs = -1;
 
@@ -719,14 +790,13 @@ bool CHMFile::InfoFromSystem()
 		if(index > size - 1 - (long)sizeof(u_int16_t))
 			break;
 
-		cursor = (u_int16_t *)(buffer + index);
-		FIXENDIAN16(*cursor);
-		
-		switch(*cursor) {
+		cursor = buffer + index;
+		value = UINT16ARRAY(cursor);
+
+		switch(value) {
 		case 0:
 			index += 2;
-			cursor = (u_int16_t *)(buffer + index);
-			FIXENDIAN16(*cursor);			
+			cursor = buffer + index;
 			
 			if(_topicsFile.IsEmpty())
 				_topicsFile = wxString(wxT("/")) 
@@ -735,8 +805,7 @@ bool CHMFile::InfoFromSystem()
 			break;
 		case 1:
 			index += 2;
-			cursor = (u_int16_t *)(buffer + index);
-			FIXENDIAN16(*cursor);			
+			cursor = buffer + index;
 
 			if(_indexFile.IsEmpty())
 				_indexFile = wxString(wxT("/"))
@@ -745,8 +814,7 @@ bool CHMFile::InfoFromSystem()
 			break;
 		case 2:
 			index += 2;
-			cursor = (u_int16_t *)(buffer + index);
-			FIXENDIAN16(*cursor);			
+			cursor = buffer + index;
 
 			if(_home.IsEmpty() || _home == wxString(wxT("/")))
 				_home = wxString(wxT("/"))
@@ -755,8 +823,7 @@ bool CHMFile::InfoFromSystem()
 			break;
 		case 3:
 			index += 2;
-			cursor = (u_int16_t *)(buffer + index);
-			FIXENDIAN16(*cursor);			
+			cursor = buffer + index;
 
 			if(_title.IsEmpty())
 				_title = CURRENT_CHAR_STRING(buffer + 
@@ -764,8 +831,7 @@ bool CHMFile::InfoFromSystem()
 			break;
 		case 6:
 			index += 2;
-			cursor = (u_int16_t *)(buffer + index);
-			FIXENDIAN16(*cursor);			
+			cursor = buffer + index;
 
 			if(_topicsFile.IsEmpty()) {
 				wxString topicAttempt = wxT("/"), tmp;
@@ -791,8 +857,7 @@ bool CHMFile::InfoFromSystem()
 
 		case 16:
 			index += 2;
-			cursor = (u_int16_t *)(buffer + index);
-			FIXENDIAN16(*cursor);			
+			cursor = buffer + index;
 
 			_font = CURRENT_CHAR_STRING(buffer + index + 2);
 			_font.AfterLast(wxT(',')).ToLong(&cs);
@@ -801,11 +866,11 @@ bool CHMFile::InfoFromSystem()
 			break;
 		default:
 			index += 2;
-			cursor = (u_int16_t *)(buffer + index);
-			FIXENDIAN16(*cursor);			
+			cursor = buffer + index;
 		}
 
-		index += *cursor + 2;
+		value = UINT16ARRAY(cursor);
+		index += value + 2;
 	}
 
 	return true;
