@@ -1,6 +1,8 @@
 /*
 
   Copyright (C) 2003  Razvan Cojocaru <razvanco@gmx.net>
+   XML-RPC/Context ID code contributed by Eamon Millman / PCI Geomatics
+  <millman@pcigeomatics.com>
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -74,9 +76,11 @@ const wxChar *greeting = wxT(
 
 
 const wxChar *about_txt = wxT(
-	"xCHM v. " VERSION "\nby Razvan Cojocaru (razvanco@gmx.net)\n\n"
+	"xCHM v. " VERSION "\nby Razvan Cojocaru <razvanco@gmx.net>\n\n"
 	"With thanks to Pabs (http://bonedaddy.net/pabs3/hhm/).\n"
 	"Based on Jed Wing's CHMLIB (http://66.93.236.84/~jedwin/projects/).\n"
+	"XMLRPC code for context sensitive help contributed by\n"
+	"Eamon Millman <millman@pcigeomatics.com>.\n"
 	"Written with wxWidgets (http://www.wxwidgets.org).\n\n"
 	"This program is (proudly) under the GPL.");
 
@@ -130,7 +134,7 @@ CHMFrame::CHMFrame(const wxString& title, const wxString& booksDir,
 
 	wxPanel* temp = CreateContentsPanel();
 
-	_html = new CHMHtmlWindow(_sw, _tcl);
+	_html = new CHMHtmlWindow(_sw, _tcl, this);
 	_html->SetRelatedFrame(this, wxT("xCHM v. " VERSION));
 	_html->SetRelatedStatusBar(0);
 
@@ -273,7 +277,7 @@ void CHMFrame::OnShowContents(wxCommandEvent& WXUNUSED(event))
 		_nb->Show(FALSE);
 	} else {		
 			
-		if(_tcl->GetCount() > 1) {
+		if(_tcl->GetCount() >= 1) {
 			
 			_tb->ToggleTool(ID_Contents, TRUE);
 			_menuFile->Check(ID_Contents, TRUE);
@@ -441,19 +445,44 @@ void CHMFrame::OnCloseWindow(wxCloseEvent& WXUNUSED(event))
 }
 
 
-void CHMFrame::LoadCHM(const wxString& archive)
+bool CHMFrame::LoadCHM(const wxString& archive)
+{
+	wxBusyCursor bc;
+	bool rtn;
+	SaveBookmarks();
+	rtn = _html->LoadPage(wxString(wxT("file:")) + archive +
+		      wxT("#chm:/"));
+
+	UpdateCHMInfo();
+	LoadBookmarks();
+
+	return rtn;
+}
+
+
+bool CHMFrame::LoadContextID( const int contextID )
 {
 	wxBusyCursor bc;
 
+	CHMFile *chmf = CHMInputStream::GetCache();
+
+	if(!chmf)
+		return FALSE;
+
+	if( !chmf->IsValidCID( contextID ) )
+		return FALSE;
+
+	return _html->LoadPage(wxString(wxT("file:")) + chmf->ArchiveName()
+		+ wxT("#chm:") + chmf->GetPageByCID( contextID ) );
+}
+
+
+void CHMFrame::UpdateCHMInfo()
+{
 #if !wxUSE_UNICODE
 	static bool noSpecialFont = true;
 	static wxFontEncoding enc = wxFont::GetDefaultEncoding();
 #endif
-
-	SaveBookmarks();
-	_html->LoadPage(wxString(wxT("file:")) + archive +
-		      wxT("#chm:/"));
-
 	CHMFile *chmf = CHMInputStream::GetCache();
 
 	if(!chmf)
@@ -477,6 +506,9 @@ void CHMFrame::LoadCHM(const wxString& archive)
 		_tcl->DeleteAllItems();
 	chmf->GetTopicsTree(_tcl);
 	chmf->GetIndex(_cip->GetResultsList());
+ 
+	// call the dirty method
+	_cip->GetResultsList()->ListDirty();
 
 	if(!title.IsEmpty()) {
 		wxString titleBarText = 
@@ -542,7 +574,7 @@ void CHMFrame::LoadCHM(const wxString& archive)
 #endif
 
 	// if we have contents..
-	if(_tcl->GetCount() > 1) {		
+	if(_tcl->GetCount() >= 1) {		
 		if(!_sw->IsSplit()) {
 			_nb->Show(TRUE);
 			_sw->SplitVertically(_nb, _html, _sashPos);
@@ -560,9 +592,7 @@ void CHMFrame::LoadCHM(const wxString& archive)
 	}
 
 	// select Contents
-	_nb->SetSelection(0);
-
-	LoadBookmarks();
+	_nb->SetSelection(0);	
 }
 
 
